﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_fibonacci_word_fractal
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Directions. Oriented so adding 1 turns right.
        private enum Directions
        {
            Up,
            Right,
            Down,
            Left,
        };

        private void btnGo_Click(object sender, EventArgs e)
        {
            // Get the desired length.
            int term_num = int.Parse(txtNumTerms.Text);
            if (term_num > 30)
            {
                if (MessageBox.Show(
                    "This may take a long time. Continue anyway?",
                    "Continue?", MessageBoxButtons.YesNo)
                        == DialogResult.No)
                            return;
            }

            // Get the Fibonacci word.
            string word = FiboWord(term_num);
            char[] chars = word.ToCharArray();

            // Calculate the points.
            Point[] points = new Point[word.Length + 1];
            points[0] = new Point(0, 0);
            Directions dir = Directions.Up;
            for (int i = 0; i < word.Length; i++)
            {
                AddPoint(chars, points, i, ref dir);
            }
            Console.WriteLine("# Chars: " + chars.Length);
            Console.WriteLine("# Points: " + points.Length);

            // Find the points' bounds.
            Rectangle rect = GetBounds(points);

            // Draw.
            int wid = picDrawing.ClientSize.Width;
            int hgt = picDrawing.ClientSize.Height;
            Bitmap bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                // Scale to fit.
                RectangleF target_rect = new RectangleF(5, 5, wid - 10, hgt - 10);
                MapDrawing(gr, rect, target_rect, false);

                // Draw.
                gr.DrawLines(Pens.Black, points);
            }

            // Display the result.
            picDrawing.Image = bm;
        }

        // Calculate the nth Fibonacci word.
        private string FiboWord(int n)
        {
            // See how long it should be.
            int length = Fibo(n);
            //string s2 = "0";        // s - 2
            //string s1 = "01";       // s - 1 
            string s2 = "0";        // s - 2
            string s1 = "1";        // s - 1 
            string s0 = "010";      // s (Initially s = 2.)
            while (s0.Length < length)
            {
                s0 = s1 + s2;
                s2 = s1;
                s1 = s0;
            }
            return s0.Substring(0, length);
        }

        // Return the nth Fibonacci number.
        private int Fibo(int n)
        {
            if (n <= 1) return n;
            int n2 = 0;         // n - 2
            int n1 = 1;         // n - 1
            int n0 = n1 + n2;   // n (Initially n = 2.)
            for (int i = 2; i < n; i++)
            {
                n2 = n1;
                n1 = n0;
                n0 = n1 + n2;
            }
            return n0;
        }

        // Add the next point in the indicated direction.
        private void AddPoint(char[] chars, Point[] points, int i, ref Directions dir)
        {
            // If the character is odd, turn 90 degrees.
            if (chars[i] == '1')
            {
                if (i % 2 == 0)
                    dir = (Directions)(((int)dir + 1) % 4);
                else
                    dir = (Directions)(((int)dir + 3) % 4);
            }

            // Find the next point.
            const int dist = 5;
            int x = points[i].X;
            int y = points[i].Y;
            if (dir == Directions.Up) y -= dist;
            else if (dir == Directions.Down) y += dist;
            else if (dir == Directions.Left) x -= dist;
            else if (dir == Directions.Right) x += dist;
            points[i + 1] = new Point(x, y);
        }

        // Find the bounds of the points.
        private Rectangle GetBounds(Point[] points)
        {
            int xmin = points[0].X;
            int xmax = xmin;
            int ymin = points[0].Y;
            int ymax = ymin;
            foreach (Point point in points)
            {
                if (point.X < xmin) xmin = point.X;
                if (point.X > xmax) xmax = point.X;
                if (point.Y < ymin) ymin = point.Y;
                if (point.Y > ymax) ymax = point.Y;
            }
            return new Rectangle(xmin, ymin, xmax - xmin, ymax - ymin);
        }

        // Map a drawing coordinate rectangle to
        // a graphics object rectangle.
        private void MapDrawing(Graphics gr, RectangleF drawing_rect,
            RectangleF target_rect, bool stretch)
        {
            gr.ResetTransform();

            // Center the drawing area at the origin.
            float drawing_cx = (drawing_rect.Left + drawing_rect.Right) / 2;
            float drawing_cy = (drawing_rect.Top + drawing_rect.Bottom) / 2;
            gr.TranslateTransform(-drawing_cx, -drawing_cy);

            // Scale.
            // Get scale factors for both directions.
            float scale_x = target_rect.Width / drawing_rect.Width;
            float scale_y = target_rect.Height / drawing_rect.Height;
            if (!stretch)
            {
                // To preserve the aspect ratio,
                // use the smaller scale factor.
                scale_x = Math.Min(scale_x, scale_y);
                scale_y = scale_x;
            }
            gr.ScaleTransform(scale_x, scale_y, MatrixOrder.Append);

            // Translate to center over the drawing area.
            float graphics_cx = (target_rect.Left + target_rect.Right) / 2;
            float graphics_cy = (target_rect.Top + target_rect.Bottom) / 2;
            gr.TranslateTransform(graphics_cx, graphics_cy, MatrixOrder.Append);
        }
    }
}
